@@ -2,39 +2,39 @@
 # A lot of other brmsfit methods have their own dedicated files.
 
 #' Extract Population-Level Estimates
-#' 
-#' Extract the population-level ('fixed') effects 
-#' from a \code{brmsfit} object. 
-#' 
+#'
+#' Extract the population-level ('fixed') effects
+#' from a \code{brmsfit} object.
+#'
 #' @aliases fixef
-#' 
+#'
 #' @inheritParams predict.brmsfit
 #' @param pars Optional names of coefficients to extract.
 #'   By default, all coefficients are extracted.
 #' @param ... Currently ignored.
-#' 
+#'
 #' @return If \code{summary} is \code{TRUE}, a matrix returned
 #'   by \code{\link{posterior_summary}} for the population-level effects.
-#'   If \code{summary} is \code{FALSE}, a matrix with one row per 
+#'   If \code{summary} is \code{FALSE}, a matrix with one row per
 #'   posterior draw and one column per population-level effect.
-#' 
+#'
 #' @examples
 #' \dontrun{
-#' fit <- brm(time | cens(censored) ~ age + sex + disease, 
+#' fit <- brm(time | cens(censored) ~ age + sex + disease,
 #'            data = kidney, family = "exponential")
 #' fixef(fit)
 #' # extract only some coefficients
 #' fixef(fit, pars = c("age", "sex"))
 #' }
-#' 
+#'
 #' @method fixef brmsfit
 #' @export
 #' @export fixef
 #' @importFrom nlme fixef
-fixef.brmsfit <-  function(object, summary = TRUE, robust = FALSE, 
+fixef.brmsfit <-  function(object, summary = TRUE, robust = FALSE,
                            probs = c(0.025, 0.975), pars = NULL, ...) {
-  contains_samples(object)
-  all_pars <- parnames(object)
+  contains_draws(object)
+  all_pars <- variables(object)
   fpars <- all_pars[grepl(fixef_pars(), all_pars)]
   if (!is.null(pars)) {
     pars <- as.character(pars)
@@ -43,7 +43,7 @@ fixef.brmsfit <-  function(object, summary = TRUE, robust = FALSE,
   if (!length(fpars)) {
     return(NULL)
   }
-  out <- as.matrix(object, pars = fpars, fixed = TRUE)
+  out <- as.matrix(object, variable = fpars)
   colnames(out) <- gsub(fixef_pars(), "", fpars)
   if (summary) {
     out <- posterior_summary(out, probs, robust)
@@ -52,30 +52,30 @@ fixef.brmsfit <-  function(object, summary = TRUE, robust = FALSE,
 }
 
 #' Covariance and Correlation Matrix of Population-Level Effects
-#' 
-#' Get a point estimate of the covariance or 
+#'
+#' Get a point estimate of the covariance or
 #' correlation matrix of population-level parameters
-#' 
+#'
 #' @inheritParams fixef.brmsfit
-#' @param correlation Logical; if \code{FALSE} (the default), compute 
+#' @param correlation Logical; if \code{FALSE} (the default), compute
 #'   the covariance matrix, if \code{TRUE}, compute the correlation matrix.
-#' 
+#'
 #' @return covariance or correlation matrix of population-level parameters
-#' 
-#' @details Estimates are obtained by calculating the maximum likelihood 
-#'   covariances (correlations) of the posterior samples. 
-#'   
+#'
+#' @details Estimates are obtained by calculating the maximum likelihood
+#'   covariances (correlations) of the posterior draws.
+#'
 #' @examples
 #' \dontrun{
-#' fit <- brm(count ~ zAge + zBase * Trt + (1+Trt|visit), 
+#' fit <- brm(count ~ zAge + zBase * Trt + (1+Trt|visit),
 #'            data = epilepsy, family = gaussian(), chains = 2)
 #' vcov(fit)
 #' }
 #'
 #' @export
 vcov.brmsfit <- function(object, correlation = FALSE, pars = NULL, ...) {
-  contains_samples(object)
-  all_pars <- parnames(object)
+  contains_draws(object)
+  all_pars <- variables(object)
   fpars <- all_pars[grepl(fixef_pars(), all_pars)]
   if (!is.null(pars)) {
     pars <- as.character(pars)
@@ -84,58 +84,58 @@ vcov.brmsfit <- function(object, correlation = FALSE, pars = NULL, ...) {
   if (!length(fpars)) {
     return(NULL)
   }
-  samples <- posterior_samples(object, pars = fpars, fixed = TRUE)
-  names(samples) <- sub(fixef_pars(), "", names(samples))
+  draws <- as.data.frame(object, variable = fpars)
+  names(draws) <- sub(fixef_pars(), "", names(draws))
   if (correlation) {
-    out <- cor(samples) 
+    out <- cor(draws)
   } else {
-    out <- cov(samples)
+    out <- cov(draws)
   }
   out
 }
 
 #' Extract Group-Level Estimates
-#' 
-#' Extract the group-level ('random') effects of each level 
-#' from a \code{brmsfit} object. 
-#' 
+#'
+#' Extract the group-level ('random') effects of each level
+#' from a \code{brmsfit} object.
+#'
 #' @aliases ranef
-#' 
+#'
 #' @inheritParams fixef.brmsfit
 #' @param groups Optional names of grouping variables
 #'   for which to extract effects.
 #' @param ... Currently ignored.
 #'
-#' @return A list of 3D arrays (one per grouping factor). 
-#'  If \code{summary} is \code{TRUE}, 
-#'  the 1st dimension contains the factor levels, 
-#'  the 2nd dimension contains the summary statistics 
+#' @return A list of 3D arrays (one per grouping factor).
+#'  If \code{summary} is \code{TRUE},
+#'  the 1st dimension contains the factor levels,
+#'  the 2nd dimension contains the summary statistics
 #'  (see \code{\link{posterior_summary}}), and
-#'  the 3rd dimension contains the group-level effects. 
+#'  the 3rd dimension contains the group-level effects.
 #'  If \code{summary} is \code{FALSE}, the 1st dimension contains
-#'  the posterior draws, the 2nd dimension contains the factor levels, 
-#'  and the 3rd dimension contains the group-level effects. 
-#'  
+#'  the posterior draws, the 2nd dimension contains the factor levels,
+#'  and the 3rd dimension contains the group-level effects.
+#'
 #' @examples
 #' \dontrun{
-#' fit <- brm(count ~ zAge + zBase * Trt + (1+Trt|visit), 
+#' fit <- brm(count ~ zAge + zBase * Trt + (1+Trt|visit),
 #'            data = epilepsy, family = gaussian(), chains = 2)
 #' ranef(fit)
 #' }
-#' 
+#'
 #' @method ranef brmsfit
 #' @export
 #' @export ranef
 #' @importFrom nlme ranef
 ranef.brmsfit <- function(object, summary = TRUE, robust = FALSE,
-                          probs = c(0.025, 0.975), pars = NULL, 
+                          probs = c(0.025, 0.975), pars = NULL,
                           groups = NULL, ...) {
-  contains_samples(object)
+  contains_draws(object)
   object <- restructure(object)
   if (!nrow(object$ranef)) {
     stop2("The model does not contain group-level effects.")
   }
-  all_pars <- parnames(object)
+  all_pars <- variables(object)
   if (!is.null(pars)) {
     pars <- as.character(pars)
   }
@@ -159,54 +159,60 @@ ranef.brmsfit <- function(object, summary = TRUE, robust = FALSE,
       regex <- paste0(",", regex, "\\]$")
       rpars <- rpars[grepl(regex, rpars)]
     }
-    out[[g]] <- as.matrix(object, rpars, fixed = TRUE)
     levels <- attr(ranef, "levels")[[g]]
-    dim(out[[g]]) <- c(nrow(out[[g]]), length(levels), length(coefs))
+    if (length(rpars)) {
+      # draws of varying coefficients were saved
+      out[[g]] <- as.matrix(object, variable = rpars)
+      dim(out[[g]]) <- c(nrow(out[[g]]), length(levels), length(coefs))
+    } else {
+      # draws of varying coefficients were not saved
+      out[[g]] <- array(dim = c(ndraws(object), length(levels), length(coefs)))
+    }
     dimnames(out[[g]])[2:3] <- list(levels, coefs)
     if (summary) {
       out[[g]] <- posterior_summary(out[[g]], probs, robust)
     }
   }
   rmNULL(out, recursive = FALSE)
-} 
+}
 
 #' Extract Model Coefficients
 #'
-#' Extract model coefficients, which are the sum of population-level 
+#' Extract model coefficients, which are the sum of population-level
 #' effects and corresponding group-level effects
-#' 
+#'
 #' @inheritParams ranef.brmsfit
 #' @param ... Further arguments passed to \code{\link{fixef.brmsfit}}
 #'   and \code{\link{ranef.brmsfit}}.
 #'
-#' @return A list of 3D arrays (one per grouping factor). 
-#'  If \code{summary} is \code{TRUE}, 
-#'  the 1st dimension contains the factor levels, 
-#'  the 2nd dimension contains the summary statistics 
+#' @return A list of 3D arrays (one per grouping factor).
+#'  If \code{summary} is \code{TRUE},
+#'  the 1st dimension contains the factor levels,
+#'  the 2nd dimension contains the summary statistics
 #'  (see \code{\link{posterior_summary}}), and
-#'  the 3rd dimension contains the group-level effects. 
+#'  the 3rd dimension contains the group-level effects.
 #'  If \code{summary} is \code{FALSE}, the 1st dimension contains
-#'  the posterior draws, the 2nd dimension contains the factor levels, 
-#'  and the 3rd dimension contains the group-level effects. 
-#'  
+#'  the posterior draws, the 2nd dimension contains the factor levels,
+#'  and the 3rd dimension contains the group-level effects.
+#'
 #' @examples
 #' \dontrun{
-#' fit <- brm(count ~ zAge + zBase * Trt + (1+Trt|visit), 
+#' fit <- brm(count ~ zAge + zBase * Trt + (1+Trt|visit),
 #'            data = epilepsy, family = gaussian(), chains = 2)
 #' ## extract population and group-level coefficients separately
 #' fixef(fit)
 #' ranef(fit)
-#' ## extract combined coefficients 
+#' ## extract combined coefficients
 #' coef(fit)
 #' }
-#' 
+#'
 #' @export
 coef.brmsfit <- function(object, summary = TRUE, robust = FALSE,
                          probs = c(0.025, 0.975), ...) {
-  contains_samples(object)
+  contains_draws(object)
   object <- restructure(object)
   if (!nrow(object$ranef)) {
-    stop2("No group-level effects detected. Call method ", 
+    stop2("No group-level effects detected. Call method ",
           "'fixef' to access population-level effects.")
   }
   fixef <- fixef(object, summary = FALSE, ...)
@@ -230,7 +236,7 @@ coef.brmsfit <- function(object, summary = TRUE, robust = FALSE,
   rm_fixef <- fixef_names %in% miss_fixef_no_digits
   fixef <- fixef[, !rm_fixef, drop = FALSE]
   fixef <- do_call(cbind, c(list(fixef), rmNULL(new_fixef)))
-  
+
   for (g in names(coef)) {
     # add missing coefficients to ranef
     ranef_names <- dimnames(coef[[g]])[[3]]
@@ -257,14 +263,14 @@ coef.brmsfit <- function(object, summary = TRUE, robust = FALSE,
         resp <- if (is_mv(object)) get_matches("^[^_]+", nm)
         family <- family(object, resp = resp)$family
         if (has_thres_minus_eta(family)) {
-          coef[[g]][, , nm] <- fixef[, nm] - coef[[g]][, , nm] 
+          coef[[g]][, , nm] <- fixef[, nm] - coef[[g]][, , nm]
         } else if (has_eta_minus_thres(family)) {
           coef[[g]][, , nm] <- coef[[g]][, , nm] - fixef[, nm]
         } else {
-          coef[[g]][, , nm] <- fixef[, nm] + coef[[g]][, , nm] 
+          coef[[g]][, , nm] <- fixef[, nm] + coef[[g]][, , nm]
         }
       } else {
-        coef[[g]][, , nm] <- fixef[, nm] + coef[[g]][, , nm] 
+        coef[[g]][, , nm] <- fixef[, nm] + coef[[g]][, , nm]
       }
     }
     if (summary) {
@@ -275,33 +281,33 @@ coef.brmsfit <- function(object, summary = TRUE, robust = FALSE,
 }
 
 #' Extract Variance and Correlation Components
-#' 
-#' This function calculates the estimated standard deviations, 
-#' correlations and covariances of the group-level terms 
-#' in a multilevel model of class \code{brmsfit}. 
-#' For linear models, the residual standard deviations, 
-#' correlations and covariances are also returned. 
-#' 
+#'
+#' This function calculates the estimated standard deviations,
+#' correlations and covariances of the group-level terms
+#' in a multilevel model of class \code{brmsfit}.
+#' For linear models, the residual standard deviations,
+#' correlations and covariances are also returned.
+#'
 #' @aliases VarCorr
-#' 
-#' @param x An object of class \code{brmsfit}. 
+#'
+#' @param x An object of class \code{brmsfit}.
 #' @inheritParams fixef.brmsfit
-#' @param sigma Ignored (included for compatibility with 
+#' @param sigma Ignored (included for compatibility with
 #'  \code{\link[nlme:VarCorr]{VarCorr}}).
 #' @param ... Currently ignored.
-#' 
+#'
 #' @return A list of lists (one per grouping factor), each with
-#' three elements: a matrix containing the standard deviations, 
-#' an array containing the correlation matrix, and an array 
+#' three elements: a matrix containing the standard deviations,
+#' an array containing the correlation matrix, and an array
 #' containing the covariance matrix with variances on the diagonal.
-#' 
+#'
 #' @examples
 #' \dontrun{
-#' fit <- brm(count ~ zAge + zBase * Trt + (1+Trt|visit), 
+#' fit <- brm(count ~ zAge + zBase * Trt + (1+Trt|visit),
 #'            data = epilepsy, family = gaussian(), chains = 2)
 #' VarCorr(fit)
 #' }
-#' 
+#'
 #' @method VarCorr brmsfit
 #' @import abind abind
 #' @importFrom nlme VarCorr
@@ -309,20 +315,20 @@ coef.brmsfit <- function(object, summary = TRUE, robust = FALSE,
 #' @export
 VarCorr.brmsfit <- function(x, sigma = 1, summary = TRUE, robust = FALSE,
                             probs = c(0.025, 0.975), ...) {
-  contains_samples(x)
+  contains_draws(x)
   x <- restructure(x)
-  if (!(nrow(x$ranef) || any(grepl("^sigma($|_)", parnames(x))))) {
+  if (!(nrow(x$ranef) || any(grepl("^sigma($|_)", variables(x))))) {
     stop2("The model does not contain covariance matrices.")
   }
   .VarCorr <- function(y) {
-    # extract samples for sd, cor and cov
-    out <- list(sd = as.matrix(x, pars = y$sd_pars, fixed = TRUE))
+    # extract draws for sd, cor and cov
+    out <- list(sd = as.matrix(x, variable = y$sd_pars))
     colnames(out$sd) <- y$rnames
     # compute correlation and covariance matrices
-    found_cor_pars <- intersect(y$cor_pars, parnames(x))
+    found_cor_pars <- intersect(y$cor_pars, variables(x))
     if (length(found_cor_pars)) {
-      cor <- as.matrix(x, pars = found_cor_pars, fixed = TRUE)
-      if (length(found_cor_pars) < length(y$cor_pars)) { 
+      cor <- as.matrix(x, variable = found_cor_pars)
+      if (length(found_cor_pars) < length(y$cor_pars)) {
         # some correlations are missing and will be replaced by 0
         cor_all <- matrix(0, nrow = nrow(cor), ncol = length(y$cor_pars))
         names(cor_all) <- y$cor_pars
@@ -348,7 +354,7 @@ VarCorr.brmsfit <- function(x, sigma = 1, summary = TRUE, robust = FALSE,
     }
     return(out)
   }
-  
+
   if (nrow(x$ranef)) {
     get_names <- function(group) {
       # get names of group-level parameters
@@ -393,12 +399,14 @@ VarCorr.brmsfit <- function(x, sigma = 1, summary = TRUE, robust = FALSE,
 
 #' @export
 model.frame.brmsfit <- function(formula, ...) {
-  formula$data 
+  formula$data
 }
 
-#' Number of Posterior Samples
+#' (Deprecated) Number of Posterior Samples
 #'
-#' Extract the number of posterior samples stored in a fitted Bayesian model.
+#' Extract the number of posterior samples (draws) stored in a fitted Bayesian
+#' model. Method \code{nsamples} is deprecated. Please use \code{ndraws}
+#' instead.
 #'
 #' @aliases nsamples
 #'
@@ -415,6 +423,7 @@ model.frame.brmsfit <- function(formula, ...) {
 #' @importFrom rstantools nsamples
 nsamples.brmsfit <- function(object, subset = NULL,
                              incl_warmup = FALSE, ...) {
+  warning2("'nsamples.brmsfit' is deprecated. Please use 'ndraws' instead.")
   if (!is(object$fit, "stanfit") || !length(object$fit@sim)) {
     out <- 0
   } else {
@@ -448,17 +457,17 @@ nobs.brmsfit <- function(object, resp = NULL, ...) {
 }
 
 #' Number of Grouping Factor Levels
-#' 
+#'
 #' Extract the number of levels of one or more grouping factors.
-#' 
+#'
 #' @aliases ngrps.brmsfit
-#' 
+#'
 #' @param object An \R object.
 #' @param ... Currently ignored.
-#' 
+#'
 #' @return A named list containing the number of levels per
 #'   grouping factor.
-#'   
+#'
 #' @export
 ngrps.brmsfit <- function(object, ...) {
   object <- restructure(object)
@@ -487,13 +496,13 @@ getCall.brmsfit <- function(x, ...) {
 }
 
 #' Extract Model Family Objects
-#' 
+#'
 #' @inheritParams posterior_predict.brmsfit
 #' @param ... Currently unused.
-#' 
+#'
 #' @return A \code{brmsfamily} object
 #' or a list of such objects for multivariate models.
-#' 
+#'
 #' @export
 family.brmsfit <- function(object, resp = NULL, ...) {
   resp <- validate_resp(resp, object)
@@ -514,32 +523,42 @@ family.brmsfit <- function(object, resp = NULL, ...) {
 }
 
 #' Expose user-defined \pkg{Stan} functions
-#' 
+#'
 #' Export user-defined \pkg{Stan} function and
-#' optionally vectorize them. For more details see 
+#' optionally vectorize them. For more details see
 #' \code{\link[rstan:expose_stan_functions]{expose_stan_functions}}.
-#' 
+#'
 #' @param x An object of class \code{brmsfit}.
 #' @param vectorize Logical; Indicates if the exposed functions
-#'   should be vectorized via \code{\link{Vectorize}}. 
+#'   should be vectorized via \code{\link{Vectorize}}.
 #'   Defaults to \code{FALSE}.
 #' @param env Environment where the functions should be made
 #'   available. Defaults to the global environment.
-#' @param ... Further arguments passed to 
+#' @param ... Further arguments passed to
 #'   \code{\link[rstan:expose_stan_functions]{expose_stan_functions}}.
-#' 
+#'
 #' @export
-expose_functions.brmsfit <- function(x, vectorize = FALSE, 
+expose_functions.brmsfit <- function(x, vectorize = FALSE,
                                      env = globalenv(), ...) {
   vectorize <- as_one_logical(vectorize)
+  if (x$backend == "cmdstanr") {
+    # cmdstanr does not yet support 'expose_stan_functions' itself (#1176)
+    scode  <- strsplit(stancode(x), "\n")[[1]]
+    data_line <- grep("^data[ ]+\\{$", scode)
+    scode <- paste0(c(scode[seq_len(data_line - 1)], "\n"), collapse = "\n")
+    stanmodel <- tempfile(fileext = ".stan")
+    cat(scode, file = stanmodel)
+  } else {
+    stanmodel <- x$fit
+  }
   if (vectorize) {
-    funs <- rstan::expose_stan_functions(x$fit, env = environment(), ...)
+    funs <- rstan::expose_stan_functions(stanmodel, env = environment(), ...)
     for (i in seq_along(funs)) {
       FUN <- Vectorize(get(funs[i], mode = "function"))
-      assign(funs[i], FUN, pos = env) 
+      assign(funs[i], FUN, pos = env)
     }
   } else {
-    funs <- rstan::expose_stan_functions(x$fit, env = env, ...)
+    funs <- rstan::expose_stan_functions(stanmodel, env = env, ...)
   }
   invisible(funs)
 }
