@@ -6,13 +6,15 @@
 #'
 #' @name StudentT
 #'
-#' @param x,q Vector of quantiles.
+#' @param x Vector of quantiles.
+#' @param q Vector of quantiles.
 #' @param p Vector of probabilities.
 #' @param n Number of draws to sample from the distribution.
 #' @param mu Vector of location values.
 #' @param sigma Vector of scale values.
 #' @param df Vector of degrees of freedom.
-#' @param log,log.p Logical; If \code{TRUE}, values are returned on the log scale.
+#' @param log Logical; If \code{TRUE}, values are returned on the log scale.
+#' @param log.p Logical; If \code{TRUE}, values are returned on the log scale.
 #' @param lower.tail Logical; If \code{TRUE} (default), return P(X <= x).
 #'   Else, return P(X > x) .
 #'
@@ -45,10 +47,12 @@ pstudent_t <- function(q, df, mu = 0, sigma = 1,
 
 #' @rdname StudentT
 #' @export
-qstudent_t <-  function(p, df, mu = 0, sigma = 1) {
+qstudent_t <-  function(p, df, mu = 0, sigma = 1,
+                        lower.tail = TRUE, log.p = FALSE) {
   if (isTRUE(any(sigma < 0))) {
     stop2("sigma must be non-negative.")
   }
+  p <- validate_p_dist(p, lower.tail = lower.tail, log.p = log.p)
   mu + sigma * qt(p, df = df)
 }
 
@@ -331,12 +335,7 @@ qskew_normal <- function(p, mu = 0, sigma = 1, alpha = 0,
   if (isTRUE(any(sigma < 0))) {
     stop2("sigma must be non-negative.")
   }
-  if (log.p) {
-    p <- exp(p)
-  }
-  if (!lower.tail) {
-    p <- 1 - p
-  }
+  p <- validate_p_dist(p, lower.tail = lower.tail, log.p = log.p)
   args <- cp2dp(mu, sigma, alpha, xi = xi, omega = omega, p = p)
   out <- with(args, {
     # do it like sn::qsn
@@ -750,21 +749,13 @@ pfrechet <- function(q, loc = 0, scale = 1, shape = 1,
 #' @export
 qfrechet <- function(p, loc = 0, scale = 1, shape = 1,
                      lower.tail = TRUE, log.p = FALSE) {
-  if (isTRUE(any(p <= 0)) || isTRUE(any(p >= 1))) {
-    stop("'p' must contain probabilities in (0,1)")
-  }
   if (isTRUE(any(scale <= 0))) {
     stop2("Argument 'scale' must be positive.")
   }
   if (isTRUE(any(shape <= 0))) {
     stop2("Argument 'shape' must be positive.")
   }
-  if (log.p) {
-    p <- exp(p)
-  }
-  if (!lower.tail) {
-    p <- 1 - p
-  }
+  p <- validate_p_dist(p, lower.tail = lower.tail, log.p = log.p)
   loc + scale * (-log(p))^(-1/shape)
 }
 
@@ -929,7 +920,7 @@ rinv_gaussian <- function(n, mu = 1, shape = 1) {
 #' @param size Vector of number of trials (zero or more).
 #' @param mu Vector of means.
 #' @param phi Vector of precisions.
-#' 
+#'
 #' @export
 dbeta_binomial <- function(x, size, mu, phi, log = FALSE) {
   require_package("extraDistr")
@@ -944,7 +935,7 @@ pbeta_binomial <- function(q, size, mu, phi, lower.tail = TRUE, log.p = FALSE) {
   require_package("extraDistr")
   alpha <- mu * phi
   beta <- (1 - mu) * phi
-  extraDistr::pbbinom(q, size, alpha = alpha, beta = beta, 
+  extraDistr::pbbinom(q, size, alpha = alpha, beta = beta,
                       lower.tail = lower.tail, log.p = log.p)
 }
 
@@ -1018,6 +1009,24 @@ pgen_extreme_value <- function(q, mu = 0, sigma = 1, xi = 0,
   out
 }
 
+#' @rdname GenExtremeValue
+#' @export
+qgen_extreme_value <- function(p, mu = 0, sigma = 1, xi = 0,
+                               lower.tail = TRUE, log.p = FALSE) {
+  if (isTRUE(any(sigma <= 0))) {
+    stop2("sigma bust be positive.")
+  }
+  p <- validate_p_dist(p, lower.tail = lower.tail, log.p = log.p)
+  args <- nlist(p, mu, sigma, xi)
+  args <- do_call(expand, args)
+  out <- with(args, ifelse(
+    xi == 0,
+    mu - sigma * log(-log(p)),
+    mu + (sigma * (1 - (-log(p))^xi)) / xi
+  ))
+  out
+}
+
 #' @rdname GenExtremeValue
 #' @export
 rgen_extreme_value <- function(n, mu = 0, sigma = 1, xi = 0) {
@@ -1087,12 +1096,7 @@ pasym_laplace <- function(q, mu = 0, sigma = 1, quantile = 0.5,
 #' @export
 qasym_laplace <- function(p, mu = 0, sigma = 1, quantile = 0.5,
                           lower.tail = TRUE, log.p = FALSE) {
-  if (log.p) {
-    p <- exp(p)
-  }
-  if (!lower.tail) {
-    p <- 1 - p
-  }
+  p <- validate_p_dist(p, lower.tail = lower.tail, log.p = log.p)
   if (length(quantile) == 1L) {
     quantile <- rep(quantile, length(mu))
   }
@@ -1173,12 +1177,7 @@ qdiscrete_weibull <- function(p, mu, shape, lower.tail = TRUE, log.p = FALSE) {
   if (isTRUE(any(shape <= 0))) {
     stop2("shape bust be positive.")
   }
-  if (log.p) {
-    p <- exp(p)
-  }
-  if (!lower.tail) {
-    p <- 1 - p
-  }
+  p <- validate_p_dist(p, lower.tail = lower.tail, log.p = log.p)
   ceiling((log(1 - p) / log(mu))^(1 / shape) - 1)
 }
 
@@ -2086,10 +2085,9 @@ inv_link_categorical <- function(x, refcat = 1, log = FALSE) {
   if (!is.null(refcat)) {
     x <- insert_refcat(x, refcat = refcat)
   }
-  if (log) {
-    out <- log_softmax(x)
-  } else {
-    out <- softmax(x)
+  out <- log_softmax(x)
+  if (!log) {
+    out <- exp(out)
   }
   out
 }
@@ -2540,6 +2538,20 @@ rshifted <- function(dist, n, shift = 0, ...) {
   do_call(paste0("r", dist), list(n, ...)) + shift
 }
 
+# validate argument p in q<dist> functions
+validate_p_dist <- function(p, lower.tail = TRUE, log.p = FALSE) {
+  if (log.p) {
+    p <- exp(p)
+  }
+  if (!lower.tail) {
+    p <- 1 - p
+  }
+  if (isTRUE(any(p <= 0)) || isTRUE(any(p >= 1))) {
+    stop2("'p' must contain probabilities in (0,1)")
+  }
+  p
+}
+
 # check if 'n' in r<dist> functions is valid
 # @param n number of desired random draws
 # @param .. parameter vectors
