@@ -227,7 +227,17 @@
 #'   between \code{0} and \code{1}. The default priors are flat over the
 #'   respective definition areas.
 #'
-#'   7. Distance parameters of monotonic effects
+#'   7. Parameters of measurement error terms
+#'
+#'   Latent variables induced via measurement error \code{\link{me}} terms
+#'   require both mean and standard deviation parameters, whose prior classes
+#'   are named \code{"meanme"} and \code{"sdme"}, respectively. If multiple
+#'   latent variables are induced this way, their correlation matrix will
+#'   be modeled as well and corresponding priors can be specified via the
+#'   \code{"corme"} class. All of the above parameters have flat priors over
+#'   their respective definition spaces by default.
+#'
+#'   8. Distance parameters of monotonic effects
 #'
 #'   As explained in the details section of \code{\link{brm}},
 #'   monotonic effects make use of a special parameter vector to
@@ -252,7 +262,7 @@
 #'   prior (i.e. \code{<vector> = rep(1, K-1)}) over all simplexes
 #'   of the respective dimension.
 #'
-#'   8. Parameters for specific families
+#'   9. Parameters for specific families
 #'
 #'   Some families need additional parameters to be estimated.
 #'   Families \code{gaussian}, \code{student}, \code{skew_normal},
@@ -371,6 +381,10 @@ set_prior <- function(prior, class = "b", coef = "", group = "",
       stop2("Argument 'coef' may not be specified when using boundaries.")
     }
   }
+  if (dpar == "mu") {
+    # distributional parameter 'mu' is currently implicit #1368
+    dpar <- ""
+  }
   if (!check) {
     # prior will be added to the log-posterior as is
     class <- coef <- group <- resp <- dpar <- nlpar <- lb <- ub <- ""
@@ -456,7 +470,8 @@ prior_string <- function(prior, ...) {
 #'
 #' @export
 get_prior <- function(formula, data, family = gaussian(), autocor = NULL,
-                      data2 = NULL, knots = NULL, sparse = NULL, ...) {
+                      data2 = NULL, knots = NULL, drop_unused_levels = TRUE,
+                      sparse = NULL, ...) {
   if (is.brmsfit(formula)) {
     stop2("Use 'prior_summary' to extract priors from 'brmsfit' objects.")
   }
@@ -471,7 +486,8 @@ get_prior <- function(formula, data, family = gaussian(), autocor = NULL,
   )
   data <- validate_data(
     data, bterms = bterms,
-    data2 = data2, knots = knots
+    data2 = data2, knots = knots,
+    drop_unused_levels = drop_unused_levels
   )
   .get_prior(bterms, data, ...)
 }
@@ -536,7 +552,7 @@ prior_predictor.mvbrmsterms <- function(x, internal = FALSE, ...) {
       prior <- prior + brmsprior(class = "rescor", prior = "lkj(1)")
     }
     if (family_names(x)[1] %in% "student") {
-      prior <- prior + 
+      prior <- prior +
         brmsprior(class = "nu", prior = "gamma(2, 0.1)", lb = "1")
     }
   }
@@ -572,7 +588,8 @@ prior_predictor.brmsterms <- function(x, data, internal = FALSE, ...) {
       dp_prior <- prior_predictor(
         x$dpars[[dp]], data = data,
         def_scale_prior = def_scale_prior,
-        def_dprior = def_dprior
+        def_dprior = def_dprior,
+        internal = internal
       )
     } else if (!is.null(x$fdpars[[dp]])) {
       # parameter is fixed
@@ -581,7 +598,7 @@ prior_predictor.brmsterms <- function(x, data, internal = FALSE, ...) {
       # parameter is estimated
       dp_bound <- dpar_bounds(dp, suffix = x$resp, family = x$family)
       dp_prior <- brmsprior(
-        def_dprior, class = dp, resp = x$resp, 
+        def_dprior, class = dp, resp = x$resp,
         lb = dp_bound$lb, ub = dp_bound$ub
       )
     }
@@ -592,7 +609,8 @@ prior_predictor.brmsterms <- function(x, data, internal = FALSE, ...) {
     nlp_prior <- prior_predictor(
       x$nlpars[[nlp]], data = data,
       def_scale_prior = def_scale_prior,
-      def_dprior = def_dprior
+      def_dprior = def_dprior,
+      internal = internal
     )
     prior <- prior + nlp_prior
   }
@@ -613,14 +631,6 @@ prior_predictor.brmsterms <- function(x, data, internal = FALSE, ...) {
         brmsprior("lkj(1)", class = "lncor", resp = x$resp)
     }
   }
-  # priors for noise-free response variables
-  sdy <- get_sdy(x, data)
-  if (!is.null(sdy)) {
-    prior <- prior +
-      brmsprior(class = "meanme", resp = x$resp) +
-      # don't specify lb as we already have it in 'prior_Xme'
-      brmsprior(class = "sdme", resp = x$resp)
-  }
   # priors for autocorrelation parameters
   # prior <- prior + prior_autocor(x, def_scale_prior = def_scale_prior)
   prior
@@ -644,7 +654,8 @@ prior_predictor.btl <- function(x, ...) {
 prior_predictor.btnl <- function(x, ...) {
   # thresholds are required even in non-linear ordinal models
   prior_thres(x, ...) +
-    prior_ac(x, ...)
+    prior_ac(x, ...) +
+    prior_bhaz(x, ...)
 }
 
 # priors for population-level parameters
@@ -946,8 +957,8 @@ prior_sm <- function(bterms, data, def_scale_prior, ...) {
     }
     # prior for SD parameters of the RE coefficients
     smterms <- unique(smef$term)
-    prior <- prior + 
-      brmsprior(prior = def_scale_prior, class = "sds", 
+    prior <- prior +
+      brmsprior(prior = def_scale_prior, class = "sds",
                 lb = "0", ls = px) +
       brmsprior(class = "sds", coef = smterms, ls = px)
   }
@@ -955,7 +966,7 @@ prior_sm <- function(bterms, data, def_scale_prior, ...) {
 }
 
 # priors for autocor parameters
-prior_ac <- function(bterms, def_scale_prior, ...) {
+prior_ac <- function(bterms, def_scale_prior, internal = FALSE, ...) {
   prior <- empty_prior()
   acef <- tidy_acef(bterms)
   if (!NROW(acef)) {
@@ -972,20 +983,29 @@ prior_ac <- function(bterms, def_scale_prior, ...) {
     arma_lb <- str_if(need_arma_bound, "-1")
     arma_ub <- str_if(need_arma_bound, "1")
     if (acef_arma$p > 0) {
-      prior <- prior + 
+      prior <- prior +
         brmsprior(class = "ar", ls = px, lb = arma_lb, ub = arma_ub)
     }
     if (acef_arma$q > 0) {
-      prior <- prior + 
+      prior <- prior +
         brmsprior(class = "ma", ls = px, lb = arma_lb, ub = arma_ub)
     }
   }
   if (has_ac_class(acef, "cosy")) {
     # cosy correlations may be negative in theory but
     # this causes problems with divergent transitions (#878)
-    prior <- prior + 
+    prior <- prior +
       brmsprior(class = "cosy", ls = px, lb = "0", ub = "1")
   }
+  if (has_ac_class(acef, "unstr")) {
+    if (internal) {
+      prior <- prior +
+        brmsprior("lkj_corr_cholesky(1)", class = "Lcortime", ls = px)
+    } else {
+      prior <- prior +
+        brmsprior("lkj(1)", class = "cortime", ls = px)
+    }
+  }
   if (has_ac_latent_residuals(bterms)) {
     prior <- prior +
       brmsprior(def_scale_prior, class = "sderr", ls = px, lb = "0")
@@ -995,11 +1015,11 @@ prior_ac <- function(bterms, def_scale_prior, ...) {
     sar_lb <- glue("min_eigenMsar{p}")
     sar_ub <- glue("max_eigenMsar{p}")
     if (acef_sar$type == "lag") {
-      prior <- prior + 
+      prior <- prior +
         brmsprior(class = "lagsar", lb = sar_lb, ub = sar_ub, ls = px)
     }
     if (acef_sar$type == "error") {
-      prior <- prior + 
+      prior <- prior +
         brmsprior(class = "errorsar", lb = sar_lb, ub = sar_ub, ls = px)
     }
   }
@@ -1008,7 +1028,7 @@ prior_ac <- function(bterms, def_scale_prior, ...) {
     prior <- prior +
       brmsprior(def_scale_prior, class = "sdcar", lb = "0", ls = px)
     if (acef_car$type %in% "escar") {
-      prior <- prior + 
+      prior <- prior +
         brmsprior(class = "car", lb = "0", ub = "1", ls = px)
     } else if (acef_car$type %in% "bym2") {
       prior <- prior +
@@ -1152,13 +1172,14 @@ def_scale_prior.brmsterms <- function(x, data, center = TRUE, df = 3,
 #' @export
 validate_prior <- function(prior, formula, data, family = gaussian(),
                            sample_prior = "no", data2 = NULL, knots = NULL,
-                           ...) {
+                           drop_unused_levels = TRUE, ...) {
   formula <- validate_formula(formula, data = data, family = family)
   bterms <- brmsterms(formula)
   data2 <- validate_data2(data2, bterms = bterms)
   data <- validate_data(
     data, bterms = bterms,
-    data2 = data2, knots = knots
+    data2 = data2, knots = knots,
+    drop_unused_levels = drop_unused_levels
   )
   .validate_prior(
     prior, bterms = bterms, data = data,
@@ -1185,8 +1206,8 @@ validate_prior <- function(prior, formula, data, family = gaussian(),
   prior <- prior[!no_checks, ]
   # check for duplicated priors
   prior$class <- rename(
-    prior$class, c("^cor$", "^rescor$", "^corme$", "^lncor"),
-    c("L", "Lrescor", "Lme", "Llncor"), fixed = FALSE
+    prior$class, c("^cor$", "^rescor$", "^corme$", "^lncor", "^cortime"),
+    c("L", "Lrescor", "Lme", "Llncor", "Lcortime"), fixed = FALSE
   )
   if (any(duplicated(prior))) {
     stop2("Duplicated prior specifications are not allowed.")
@@ -1208,7 +1229,7 @@ validate_prior <- function(prior, formula, data, family = gaussian(),
     prior <- prior[!invalid, ]
   }
   prior$prior <- sub("^(lkj|lkj_corr)\\(", "lkj_corr_cholesky(", prior$prior)
-  
+
   # include default parameter bounds; only new priors need bounds
   which_needs_lb <- which(is.na(prior$lb) & !nzchar(prior$coef))
   for (i in which_needs_lb) {
@@ -1238,13 +1259,13 @@ validate_prior <- function(prior, formula, data, family = gaussian(),
   }
   # the remaining NAs are in coef priors which cannot have bounds yet
   prior$lb[is.na(prior$lb)] <- prior$ub[is.na(prior$ub)] <- ""
-  
+
   # merge user-specified priors with default priors
   prior$new <- rep(TRUE, nrow(prior))
   all_priors$new <- rep(FALSE, nrow(all_priors))
   prior <- c(all_priors, prior, replace = TRUE)
   check_prior_content(prior)
-  
+
   # don't require priors on nlpars if some priors are not checked (#1124)
   require_nlpar_prior <- require_nlpar_prior && !any(no_checks)
   prior <- validate_prior_special(
@@ -1290,18 +1311,21 @@ check_prior_content <- function(prior) {
     return(invisible(TRUE))
   }
   lb_priors <- c(
-    "lognormal", "chi_square", "inv_chi_square", "scaled_inv_chi_square", 
+    "lognormal", "chi_square", "inv_chi_square", "scaled_inv_chi_square",
     "exponential", "gamma", "inv_gamma", "weibull", "frechet", "rayleigh",
     "pareto", "pareto_type_2"
   )
   lb_priors_regex <- paste0("^(", paste0(lb_priors, collapse = "|"), ")")
   ulb_priors <- c("beta", "uniform", "von_mises", "beta_proportion")
   ulb_priors_regex <- paste0("^(", paste0(ulb_priors, collapse = "|"), ")")
-  cormat_pars <- c("cor", "L", "rescor", "Lrescor", "corme", "Lme", "lncor", "Llncor")
+  cormat_pars <- c(
+    "cor", "L", "rescor", "Lrescor", "corme", "Lme",
+    "lncor", "Llncor", "cortime", "Lcortime"
+  )
   cormat_regex <- "^((lkj)|(constant))"
   simplex_pars <- c("simo", "theta", "sbhaz")
   simplex_regex <- "^((dirichlet)|(constant))\\("
-  
+
   lb_warning <- ub_warning <- ""
   for (i in seq_rows(prior)) {
     if (!nzchar(prior$prior[i]) || !prior$new[i]) {
@@ -1326,7 +1350,7 @@ check_prior_content <- function(prior) {
         "the 'lkj' prior. See help(set_prior) for more details."
       )
     }
-    if (prior$class[i] %in% simplex_pars && 
+    if (prior$class[i] %in% simplex_pars &&
         !grepl(simplex_regex, prior$prior[i])) {
       stop2(
         "Currently 'dirichlet' is the only valid prior for ",
@@ -1684,15 +1708,15 @@ convert_bounds2stan <- function(bounds, default = "") {
     stop2("Upper boundaries cannot be negative infinite.")
   }
   lb <- ifelse(
-    !is.na(lb) & !lb %in% c("NA", "-Inf", ""), 
+    !is.na(lb) & !lb %in% c("NA", "-Inf", ""),
     paste0("lower=", lb), ""
   )
   ub <- ifelse(
-    !is.na(ub) & !ub %in% c("NA", "Inf", ""), 
+    !is.na(ub) & !ub %in% c("NA", "Inf", ""),
     paste0("upper=", ub), ""
   )
   out <- ifelse(
-    nzchar(lb) & nzchar(ub), glue("<{lb},{ub}>"), 
+    nzchar(lb) & nzchar(ub), glue("<{lb},{ub}>"),
     ifelse(
       nzchar(lb) & !nzchar(ub), glue("<{lb}>"),
       ifelse(
@@ -1704,7 +1728,7 @@ convert_bounds2stan <- function(bounds, default = "") {
   out
 }
 
-# convert parameter bounds in Stan syntax 
+# convert parameter bounds in Stan syntax
 # TODO: vectorize over a character vector of bounds?
 # complicated because of a mix of character and numeric values
 # to a named list with elements 'lb' and 'ub'
@@ -1896,12 +1920,12 @@ eval_dirichlet <- function(prior, len = NULL, env = NULL) {
 #'   is greater \code{1}, the shape of the prior will no longer resemble
 #'   a horseshoe and it may be more appropriately called an hierarchical
 #'   shrinkage prior in this case.
-#' @param scale_slab Scale of the student-t prior of the regularization
-#'   parameter. Defaults to \code{2}. The original unregularized horseshoe
-#'   prior is obtained by setting \code{scale_slab} to infinite, which
-#'   we can approximate in practice by setting it to a very large real value.
-#' @param df_slab Degrees of freedom of the student-t prior of
-#'   the regularization parameter. Defaults to \code{4}.
+#' @param scale_slab Scale of the Student-t slab. Defaults to \code{2}. The
+#'   original unregularized horseshoe prior is obtained by setting
+#'   \code{scale_slab} to infinite, which we can approximate in practice by
+#'   setting it to a very large real value.
+#' @param df_slab Degrees of freedom of the student-t slab.
+#'   Defaults to \code{4}.
 #' @param par_ratio Ratio of the expected number of non-zero coefficients
 #'   to the expected number of zero coefficients. If specified,
 #'   \code{scale_global} is ignored and internally computed as
@@ -1976,7 +2000,7 @@ eval_dirichlet <- function(prior, len = NULL, env = NULL) {
 horseshoe <- function(df = 1, scale_global = 1, df_global = 1,
                       scale_slab = 2, df_slab = 4, par_ratio = NULL,
                       autoscale = TRUE) {
-  out <- deparse(match.call(), width.cutoff = 500L)
+  out <- deparse0(match.call())
   df <- as.numeric(df)
   df_global <- as.numeric(df_global)
   df_slab <- as.numeric(df_slab)
@@ -2027,7 +2051,7 @@ horseshoe <- function(df = 1, scale_global = 1, df_global = 1,
 #' @param prec_R2 precision of the Beta prior on the coefficient of determination R^2.
 #' @param cons_D2 concentration vector of the Dirichlet prior on the variance
 #'   decomposition parameters.
-#' @param autoscale Logical; indicating whether the horseshoe
+#' @param autoscale Logical; indicating whether the R2D2
 #'   prior should be scaled using the residual standard deviation
 #'   \code{sigma} if possible and sensible (defaults to \code{TRUE}).
 #'   Autoscaling is not applied for distributional parameters or
@@ -2046,7 +2070,7 @@ horseshoe <- function(df = 1, scale_global = 1, df_global = 1,
 #'
 #' @export
 R2D2 <- function(mean_R2 = 0.5, prec_R2 = 2, cons_D2 = 1, autoscale = TRUE) {
-  out <- deparse(match.call(), width.cutoff = 500L)
+  out <- deparse0(match.call())
   mean_R2 <- as_one_numeric(mean_R2)
   prec_R2 <- as_one_numeric(prec_R2)
   cons_D2 <- as.numeric(cons_D2)
@@ -2110,7 +2134,7 @@ R2D2 <- function(mean_R2 = 0.5, prec_R2 = 2, cons_D2 = 1, autoscale = TRUE) {
 #'
 #' @export
 lasso <- function(df = 1, scale = 1) {
-  out <- deparse(match.call(), width.cutoff = 500L)
+  out <- deparse0(match.call())
   df <- as.numeric(df)
   scale <- as.numeric(scale)
   if (!isTRUE(df > 0)) {
