@@ -64,7 +64,7 @@ stan_prior <- function(prior, class, coef = NULL, group = NULL,
     }
     base_prior <- base_prior[1]
     if (nrow(unique(base_bounds)) > 1L) {
-      stop2("Conflicting boundary information for ", 
+      stop2("Conflicting boundary information for ",
             "coefficients of class '", class, "'.")
     }
     base_bounds <- base_bounds[1, ]
@@ -179,6 +179,7 @@ stan_prior <- function(prior, class, coef = NULL, group = NULL,
 
   if (nzchar(type)) {
     # only define the parameter here if type is non-empty
+    type <- stan_adjust_par_type(type, base_prior)
     type <- stan_type_add_bounds(type, bound)
     comment <- stan_comment(comment)
     par_definition <- glue("  {type} {par}{dim};{comment}\n")
@@ -385,6 +386,7 @@ stan_special_prior_global <- function(bterms, data, prior, normalize, ...) {
       "  // R2D2 shrinkage parameters\n",
       "  real<lower=0,upper=1> R2D2_R2{p};  // R2 parameter\n"
     )
+    var_mult <- ""
     if (isTRUE(special$R2D2$autoscale)) {
       var_mult <- glue("sigma{usc(px$resp)}^2 * ")
     }
@@ -483,11 +485,8 @@ stan_special_prior_local <- function(prior, class, ncoef, px,
     )
   }
   if (!is.null(special$lasso)) {
-    str_add(out$par) <- glue(
-      "  vector[K{ct}{sp}] b{sp};  // population-level effects\n"
-    )
     str_add(out$model_prior) <- glue(
-      "{tp}double_exponential_lpdf(b{sp} | 0, lasso_scale{p} * lasso_inv_lambda{p});\n"
+      "{tp}double_exponential_lpdf(b{suffix}{sp} | 0, lasso_scale{p} * lasso_inv_lambda{p});\n"
     )
   }
   out
@@ -658,9 +657,30 @@ stan_type_add_bounds <- function(type, bound) {
   glue("{type_type}{bound}{type_dim}")
 }
 
+# adjust the type of a parameter based on the assigned prior
+stan_adjust_par_type <- function(type, prior) {
+  # TODO: add support for more type-prior combination?
+  combs <- data.frame(
+    type = "vector",
+    prior = "dirichlet",
+    new_type = "simplex"
+  )
+  for (i in seq_rows(combs)) {
+    regex_type <- paste0("^", combs$type[i], "\\[?")
+    regex_prior <- paste0("^", combs$prior[i], "\\(")
+    if (grepl(regex_type, type) && grepl(regex_prior, prior)) {
+      brackets <- get_matches("\\[.*\\]$", type, first = TRUE)
+      type <- paste0(combs$new_type[i], brackets)
+      break
+    }
+  }
+  type
+}
+
+# stops if a prior bound is given
 stopif_prior_bound <- function(prior, class, ...) {
-  lb <- stan_base_prior(prior, "lb", class = class, ...) 
-  ub <- stan_base_prior(prior, "ub", class = class, ...) 
+  lb <- stan_base_prior(prior, "lb", class = class, ...)
+  ub <- stan_base_prior(prior, "ub", class = class, ...)
   if (nzchar(lb) || nzchar(ub)) {
     stop2("Cannot add bounds to class '", class, "' for this prior.")
   }
