@@ -134,12 +134,12 @@ stan_response <- function(bterms, data, normalize) {
     )
   }
   if (is.formula(bterms$adforms$cens)) {
-    cens <- eval_rhs(bterms$adforms$cens)
     str_add(out$data) <- glue(
       "  int<lower=-1,upper=2> cens{resp}[N{resp}];  // indicates censoring\n"
     )
     str_add(out$pll_args) <- glue(", data int[] cens{resp}")
-    if (cens$vars$y2 != "NA") {
+    y2_expr <- get_ad_expr(bterms, "cens", "y2")
+    if (!is.null(y2_expr)) {
       # interval censoring is required
       if (rtype == "int") {
         str_add(out$data) <- glue(
@@ -413,16 +413,16 @@ stan_mixture <- function(bterms, data, prior, threads, normalize, ...) {
     missing_id <- setdiff(1:nmix, dpar_id(names(theta_pred)))
     str_add(out$model_def) <- glue(
       "  vector[N{p}] theta{missing_id}{p} = rep_vector(0.0, N{p});\n",
-      "  real log_sum_exp_theta;\n"
+      "  real log_sum_exp_theta{p};\n"
     )
     sum_exp_theta <- glue("exp(theta{1:nmix}{p}[n])", collapse = " + ")
     str_add(out$model_comp_mix) <- glue(
       "  for (n in 1:N{p}) {{\n",
       "    // scale theta to become a probability vector\n",
-      "    log_sum_exp_theta = log({sum_exp_theta});\n"
+      "    log_sum_exp_theta{p} = log({sum_exp_theta});\n"
     )
     str_add(out$model_comp_mix) <- cglue(
-      "    theta{1:nmix}{p}[n] = theta{1:nmix}{p}[n] - log_sum_exp_theta;\n"
+      "    theta{1:nmix}{p}[n] = theta{1:nmix}{p}[n] - log_sum_exp_theta{p};\n"
     )
     str_add(out$model_comp_mix) <- "  }\n"
   } else if (length(theta_fix)) {
@@ -493,7 +493,7 @@ stan_mixture <- function(bterms, data, prior, threads, normalize, ...) {
 # @return a character string
 stan_ordinal_lpmf <- function(family, link) {
   stopifnot(is.character(family), is.character(link))
-  inv_link <- stan_inv_link(link)
+  inv_link <- stan_inv_link(link, vectorize = FALSE)
   th <- function(k) {
     # helper function generating stan code inside inv_link(.)
     if (family %in% c("cumulative", "sratio")) {
@@ -663,3 +663,122 @@ stan_ordinal_lpmf <- function(family, link) {
   }
   out
 }
+
+# log probability density for hurdle ordinal models
+# @return a character string
+stan_hurdle_ordinal_lpmf <- function(family, link) {
+  stopifnot(is.character(family), is.character(link))
+  # TODO: generalize to non-cumulative families?
+  stopifnot(family == "hurdle_cumulative")
+  inv_link <- stan_inv_link(link, vectorize = FALSE)
+  th <- function(k) {
+    out <- glue("thres[{k}] - mu")
+    glue("disc * ({out})")
+  }
+  out <- glue(
+    "  /* {family}-{link} log-PDF for a single response\n",
+    "   * Args:\n",
+    "   *   y: response category\n",
+    "   *   mu: latent mean parameter\n",
+    "   *   hu: hurdle probability\n",
+    "   *   disc: discrimination parameter\n",
+    "   *   thres: ordinal thresholds\n",
+    "   * Returns:\n",
+    "   *   a scalar to be added to the log posterior\n",
+    "   */\n",
+    "   real {family}_{link}_lpmf(int y, real mu, real hu, real disc, vector thres) {{\n",
+    "\n"
+  )
+  # define the function body
+  if (inv_link == "inv_logit") {
+    str_add(out) <- glue(
+      "     int nthres = num_elements(thres);\n",
+      "     if (y == 0) {{\n",
+      "       return bernoulli_lpmf(1 | hu);\n",
+      "     }} else if (y == 1) {{\n",
+      "       return log_inv_logit({th(1)}) +\n",
+      "                bernoulli_lpmf(0 | hu);\n",
+      "     }} else if (y == nthres + 2) {{\n",
+      "       return log1m_inv_logit({th('nthres')}) +\n",
+      "                bernoulli_lpmf(0 | hu);\n",
+      "     }} else {{\n",
+      # TODO: replace with log_inv_logit_diff once rstan >= 2.25
+      "       return log_diff_exp(\n",
+      "         log_inv_logit({th('y')}), \n",
+      "         log_inv_logit({th('y - 1')})\n",
+      "       ) + bernoulli_lpmf(0 | hu) ;\n",
+      "     }}\n",
+      "   }}\n"
+    )
+  } else {
+    str_add(out) <- glue(
+      "     int nthres = num_elements(thres);\n",
+      "     real p;\n",
+      "     if (y == 0){{\n",
+      "       p = hu;\n",
+      "     }} else if (y == 1) {{\n",
+      "       p = {inv_link}({th(1)}) * (1 - hu);\n",
+      "     }} else if (y == nthres + 1) {{\n",
+      "       p = (1 - {inv_link}({th('nthres')})) * (1 - hu);\n",
+      "     }} else {{\n",
+      "       p = ({inv_link}({th('y')}) -\n",
+      "           {inv_link}({th('y - 1')})) * (1 - hu);\n",
+      "     }}\n",
+      "     return log(p);\n",
+      "   }}\n"
+    )
+  }
+
+  # lpmf function for multiple merged thresholds
+  str_add(out) <- glue(
+    "  /* {family}-{link} log-PDF for a single response and merged thresholds\n",
+    "   * Args:\n",
+    "   *   y: response category\n",
+    "   *   mu: latent mean parameter\n",
+    "   *   hu: hurdle probability\n",
+    "   *   disc: discrimination parameter\n",
+    "   *   thres: vector of merged ordinal thresholds\n",
+    "   *   j: start and end index for the applid threshold within 'thres'\n",
+    "   * Returns:\n",
+    "   *   a scalar to be added to the log posterior\n",
+    "   */\n",
+    "   real {family}_{link}_merged_lpmf(",
+    "int y, real mu, real hu, real disc, vector thres, int[] j) {{\n",
+    "     return {family}_{link}_lpmf(y | mu, hu, disc, thres[j[1]:j[2]]);\n",
+    "   }}\n"
+  )
+
+  if (link == "logit") {
+    # use the more efficient ordered_logistic function when disc == 1
+    str_add(out) <- glue(
+      "\n",
+      " // Use more efficient ordered_logistic function with disc == 1\n",
+      "   real hurdle_cumulative_ordered_logistic_lpmf(int y, real mu, real hu, real disc, vector thres) {{\n",
+      "     if (y == 0) {{\n",
+      "       return bernoulli_lpmf(1 | hu);\n",
+      "     }} else {{\n",
+      "       return ordered_logistic_lpmf(y | mu, thres) +\n",
+      "                bernoulli_lpmf(0 | hu);\n",
+      "     }}\n",
+      "   }}\n"
+    )
+    str_add(out) <- glue(
+      "  /* use ordered-logistic log-PDF for a single response and merged thresholds\n",
+      "   * Args:\n",
+      "   *   y: response category\n",
+      "   *   mu: latent mean parameter\n",
+      "   *   hu: hurdle probability\n",
+      "   *   thres: vector of merged ordinal thresholds\n",
+      "   *   j: start and end index for the applid threshold within 'thres'\n",
+      "   * Returns:\n",
+      "   *   a scalar to be added to the log posterior\n",
+      "   */\n",
+      "   real hurdle_cumulative_ordered_logistic_merged_lpmf(",
+      "int y, real mu, real hu, real disc, vector thres, int[] j) {{\n",
+      "     return hurdle_cumulative_ordered_logistic_lpmf(y | mu, hu, disc, thres[j[1]:j[2]]);\n",
+      "   }}\n"
+    )
+  }
+  out
+}
+
